package de.intevation.flys.artifacts.states.minfo;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifacts.CallContext;
import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.access.BedQualityAccess;
import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.minfo.BedBedQualityResult;
import de.intevation.flys.artifacts.model.minfo.BedDensityFacet;
import de.intevation.flys.artifacts.model.minfo.BedDiameterFacet;
import de.intevation.flys.artifacts.model.minfo.BedLoadBedQualityResult;
import de.intevation.flys.artifacts.model.minfo.BedPorosityFacet;
import de.intevation.flys.artifacts.model.minfo.BedQualityCalculation;
import de.intevation.flys.artifacts.model.minfo.BedQualityResult;
import de.intevation.flys.artifacts.model.minfo.BedloadDiameterFacet;
import de.intevation.flys.artifacts.states.DefaultState;


public class BedQualityState extends DefaultState implements FacetTypes {

    private static final long serialVersionUID = 1L;

    private static final Logger logger = Logger
        .getLogger(BedQualityState.class);

    @Override
    public Object computeAdvance(FLYSArtifact artifact, String hash,
        CallContext context, List<Facet> facets, Object old) {
        logger.debug("BedQualityState.computeAdvance");

        List<Facet> newFacets = new ArrayList<Facet>();

        BedQualityAccess access = new BedQualityAccess(artifact);

        CalculationResult res = old instanceof CalculationResult ? (CalculationResult) old
            : new BedQualityCalculation().calculate(access);

        if (facets == null || res == null) {
            return res;
        }

        BedQualityResult[] results = (BedQualityResult[]) res.getData();

        if (results == null || results.length == 0) {
            logger.warn("Calculation computed no results!");
            return res;
        }

        generateFacets(newFacets, results, getID(), hash);
        logger.debug("Created " + newFacets.size() + " new Facets.");

        facets.addAll(newFacets);

        return res;
    }

    protected void generateFacets(List<Facet> newFacets,
        BedQualityResult[] results, String stateId, String hash) {
        logger.debug("BedQualityState.generateFacets");

        for (int idx = 0; idx < results.length; idx++) {
            BedQualityResult result = results[idx];

            if (result instanceof BedBedQualityResult) {
                // TODO I18N for Facet description
                newFacets.add(new BedPorosityFacet(idx,
                    BED_QUALITY_POROSITY_TOPLAYER, "porosity_toplayer",
                    ComputeType.ADVANCE, stateId, hash));

                // TODO I18N for Facet description
                newFacets.add(new BedPorosityFacet(idx,
                    BED_QUALITY_POROSITY_SUBLAYER, "porosity_sublayer",
                    ComputeType.ADVANCE, stateId, hash));

                // TODO I18N for Facet description
                newFacets.add(new BedDensityFacet(idx,
                    BED_QUALITY_SEDIMENT_DENSITY_TOPLAYER, "density_toplayer",
                    ComputeType.ADVANCE, stateId, hash));

                // TODO I18N for Facet description
                newFacets.add(new BedDensityFacet(idx,
                    BED_QUALITY_SEDIMENT_DENSITY_SUBLAYER, "density_sublayer",
                    ComputeType.ADVANCE, stateId, hash));

                // TODO I18N for Facet description
                newFacets.add(new BedloadDiameterFacet(idx,
                    BED_QUALITY_BED_DIAMETER_TOPLAYER, "bed diameter_toplayer",
                    ComputeType.ADVANCE, stateId, hash));
                
                // TODO I18N for Facet description
                newFacets.add(new BedloadDiameterFacet(idx,
                    BED_QUALITY_BED_DIAMETER_SUBLAYER, "bed diameter_sublayer",
                    ComputeType.ADVANCE, stateId, hash));
            }
            else if (result instanceof BedLoadBedQualityResult) {
                // TODO I18N for Facet description
                newFacets.add(new BedDiameterFacet(idx, BED_QUALITY_BEDLOAD_DIAMETER,
                    "bedload diameter", ComputeType.ADVANCE, stateId, hash));
            }
            else {
                logger.warn("unknown result object: " + result.getClass());
            }
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
